import javax.sound.midi.*;

/**
 * Functional representation of a MIDI note, which adds functionality to the existent MidiEvent class.
 * Also provides a reference to the corresponding NoteOff event.
 * 
 * @author  Einar Pehrson
 */
 
public class MooNote extends MidiEvent {

	protected MidiEvent noteOffEvent;
	protected ShortMessage noteOnMsg, noteOffMsg;

	/** 
	 * Creates a MooNote from the given NoteOn event in the current track.
	 * @param noteOnEvent	the NoteOn event of the note
	 */
	public MooNote (MidiEvent noteOnEvent) {
		super(noteOnEvent.getMessage(), noteOnEvent.getTick());
		noteOnMsg = (ShortMessage)getMessage();
	}

	/** 
	 * Creates a MooNote from the given NoteOn event in the current track and creates a reference to
	 * the corresponding NoteOff event.
	 * @param noteOnEvent	the NoteOn event of the note
	 * @param noteOffEvent	the NoteOff event of the note
	 */
	public MooNote (MidiEvent noteOnEvent, MidiEvent noteOffEvent) {
		super(noteOnEvent.getMessage(), noteOnEvent.getTick());
		this.noteOffEvent = noteOffEvent;
		noteOnMsg = (ShortMessage)getMessage();
		noteOffMsg = (ShortMessage)noteOffEvent.getMessage();
	}

	/** 
	 * Creates a MooNote of the given pitch, velocity and duration in the current track.
	 * @param channel	the channel of the note (1-16)
	 * @param pitch		the pitch of the note (0-127)
	 * @param velocity	the velocity of the note (0-127)
	 * @param timestamp	the timestamp of the note in ticks (96 per beat)
	 * @param duration	the duration of the note in ticks (96 per beat)
	 */
	public MooNote (int channel, int pitch, int velocity, long timestamp, int duration) {
		super(new ShortMessage(), timestamp);
		noteOffEvent = new MidiEvent(new ShortMessage(), timestamp + duration);
		noteOnMsg = (ShortMessage)getMessage();
		noteOffMsg = (ShortMessage)noteOffEvent.getMessage();
		try {
			noteOnMsg.setMessage(ShortMessage.NOTE_ON, channel, pitch, velocity);
			noteOffMsg.setMessage(ShortMessage.NOTE_OFF, channel, pitch, 0);
		} catch (InvalidMidiDataException e) {}
	}

	/** 
	 * Sets the channel of the current note.
	 * @param channel	the channel of the note (1-16)
	 */
	public void setChannel(int channel) {
		try {
			noteOnMsg.setMessage(noteOnMsg.getCommand(), (byte)channel, noteOnMsg.getData1(), noteOnMsg.getData2());
			if(hasNoteOffEvent()) noteOffMsg.setMessage(noteOffMsg.getCommand(), (byte)channel, noteOffMsg.getData1(), noteOffMsg.getData2());
		} catch (InvalidMidiDataException e) {}
	}

	/** 
	 * Sets the pitch of the current note.
	 * @param pitch		the pitch of the note (0-127)
	 */
	public void setPitch(int pitch) {
		try {
			noteOnMsg.setMessage(noteOnMsg.getCommand(), noteOnMsg.getChannel(), (byte)pitch, noteOnMsg.getData2());
			if(hasNoteOffEvent()) noteOffMsg.setMessage(noteOffMsg.getCommand(), noteOffMsg.getChannel(), (byte)pitch, noteOffMsg.getData2());
		} catch (InvalidMidiDataException e) {}
	}

	/** 
	 * Sets the velocity of the current note.
	 * @param vel	the velocity of the note (0-127)
	 */
	public void setVelocity(int vel) {
		try {
			noteOnMsg.setMessage(noteOnMsg.getCommand(), noteOnMsg.getChannel(), noteOnMsg.getData1(), (byte)vel);
			if(hasNoteOffEvent()) noteOffMsg.setMessage(noteOffMsg.getCommand(), noteOffMsg.getChannel(), noteOffMsg.getData1(), noteOffMsg.getData2());
		} catch (InvalidMidiDataException e) {}
	}

	/** 
	 * Sets the duration of the current note (or rather moves the note off event).
	 * @param n	the duration of the note in ticks (96 per beat)
	 */
	public void setDuration(int ticks) {
		if (hasNoteOffEvent()) noteOffEvent.setTick(getTick() + ticks);
	}

	/** 
	 * Sets the timestamp of the current note.
	 * @param tick	the timestamp of the note in ticks (96 per beat)
	 */
	public void setTick(long tick) {
		if (hasNoteOffEvent()) noteOffEvent.setTick(tick +  getDuration());
		super.setTick(tick);
	}

	/** 
	 * Returns the channel of the current note.
	 * @return the channel of the note (1-16)
	 */
	public int getChannel() {
		return noteOnMsg.getChannel();
	}

	/** 
	 * Returns the pitch of the current note.
	 * @return the pitch of the note (0-127)
	 */
	public int getPitch() {
		return noteOnMsg.getData1();
	}

	/** 
	 * Returns the velocity of the current note.
	 * @return the velocity of the note (0-127)
	 */
	public int getVelocity() {
		return noteOnMsg.getData2();
	}

	/** 
	 * Returns the duration of the current note.
	 * @return the duration of the note (in ticks)
	 */
	public int getDuration() {
		if (!hasNoteOffEvent()) return 0;
		return (int)(noteOffEvent.getTick() - getTick());
	}

	/** 
	 * Returns whether the NoteOff event was found.
	 * @return	the note off MidiEvent
	 */
	public boolean hasNoteOffEvent() {
		return noteOffEvent != null;
	}

	public void addTo(Track track){
		track.add(this);
		if (hasNoteOffEvent()) track.add(noteOffEvent);
	}
	
	public void removeFrom(Track track){
		track.remove(this);
		if (hasNoteOffEvent()) track.remove(noteOffEvent);
	}
}
